﻿package  {
	import flash.display.Bitmap;
	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;

	//
	//***************************************************************
	//
	//
	// @author james@codeandvisual.com
	//
	//
	//***************************************************************
	public class EnvironmentParallax extends MovieClip{
		//
		//***************************************************************
		//
		//
		// PROPERTIES
		//
		//
		//***************************************************************
		protected var _viewport_width:int // visible display width of the environment
		protected var _viewport_height:int // visible display height of the envornmoent 
		protected var _layers:Array // array of parallax layers (Bitmaps)
		protected var _pan_target_x:Number = 0 //x location of the foreground to frame
		protected var _pan_target_y:Number = 0	//y location of the foreground to frame	
		protected var _pan_speed:Number = 30 //speed of movement
		protected var _marker:Sprite //marker to show current pan target
		//
		//***************************************************************
		//
		//
		// INIT
		//
		//
		//***************************************************************
		public function EnvironmentParallax(thisViewWidth:int, thisViewHeight:int, thisLayers:Array) {
			_viewport_width = thisViewWidth
			_viewport_height = thisViewHeight
			_layers = thisLayers
			init()
		}
		protected function init():void{
			initLayers()
			initMouse()
			initMarker()
			
		}
		//
		//***************************************************************
		//
		//
		// DISPLAY
		//
		//
		//***************************************************************
		protected function initLayers():void{
			for (var i:int = _layers.length - 1; i >= 0; i--) {
				addChild(_layers[i])
			}
		}
		//
		//***************************************************************
		//
		//
		// MARKER
		//
		//
		//***************************************************************
		protected function initMarker():void {
			//
			//------------------------------------------------------
			// Create Marker
			//------------------------------------------------------
			_marker = new Sprite()
			var mySquare:Sprite = new Sprite()
			mySquare.graphics.lineStyle(1,0xffffff,1)
			mySquare.graphics.beginFill(0xffffff,.2);
			mySquare.graphics.drawRect(0,0,20,20);
			mySquare.graphics.endFill();
			mySquare.x -= 10
			mySquare.y -= 10
			_marker.addChild(mySquare)
			addChild(_marker)
			//
			//------------------------------------------------------
			// Set marker position
			//------------------------------------------------------
			_pan_target_x = _viewport_width/2
			_pan_target_y = foreground.height - _viewport_height / 2
			panToPosition(_pan_target_x, _pan_target_y)
		}
		//
		//***************************************************************
		//
		//
		// CAMERA
		//
		//
		//***************************************************************
		public function panToPosition(thisX:Number, thisY:Number):void { 
			//
			//------------------------------------------------------
			// Store Pan Target
			//------------------------------------------------------
			_pan_target_x = clamp(thisX,0,foreground.width)
			_pan_target_y = clamp(thisY,0,foreground.height)
			//
			//------------------------------------------------------
			// Move foreground into position, clamp edges, determine pan percentage
			//------------------------------------------------------
			var myX:Number = -_pan_target_x + (_viewport_width / 2)
			var myXClamped:Number =  clamp(myX, -(foreground.width - _viewport_width), 0 )
			var myPercentX:Number = myXClamped / -(foreground.width - _viewport_width)
			var myY:Number = -_pan_target_y + (_viewport_height / 2)
			var myYClamped:Number =  clamp(myY, -(foreground.height - _viewport_height), 0 )
			var myPercentY:Number = myYClamped / -(foreground.height - _viewport_height)
			//
			//------------------------------------------------------
			// Apply pan percentage to other layers
			//------------------------------------------------------
			panToPercent(myPercentX, myPercentY)
			//
			//------------------------------------------------------
			// Update Marker Location
			//------------------------------------------------------
			_marker.x = foreground.x+_pan_target_x
			_marker.y = foreground.y+_pan_target_y
		}
		public function panToPercent(thisX:Number, thisY:Number):void {
			for (var i:int = 0; i < _layers.length; i++) {
				panLayerToPercent(_layers[i],thisX, thisY)
			}
		}
		protected function panLayerToPercent(thisLayer:Bitmap, thisXPercent:Number, thisYPercent:Number ):void {
			var myWidth:int = thisLayer.width-_viewport_width
			var myHeight:int = thisLayer.height-_viewport_height
			var myX:Number = myWidth * thisXPercent
			var myY:Number = myHeight * (thisYPercent)
			var myFramedX:Number = -myX
			var myFramedY:Number =  -myY
			thisLayer.x = myFramedX
			thisLayer.y = myFramedY
		}	
		//
		//***************************************************************
		//
		//
		// MOUSE
		//
		//
		//***************************************************************
		protected function initMouse():void {
			addEventListener(MouseEvent.MOUSE_DOWN,startMousePan)
			addEventListener(MouseEvent.MOUSE_UP,endMousePan)
		}
		protected function startMousePan(e:MouseEvent):void {
			addEventListener(Event.ENTER_FRAME,mousePan)
		}
		protected function mousePan(e:Event):void {
			var myHalfWidth:Number = _viewport_width / 2
			var myDistanceX:Number = mouseX - myHalfWidth
			var myPercentX:Number = myDistanceX/myHalfWidth
			var mySpeedX:Number = _pan_speed*myPercentX
			//
			var myHalfHeight:Number = _viewport_height / 2
			var myDistanceY:Number = mouseY -myHalfHeight
			var myPercentY:Number = myDistanceY/myHalfHeight
			var mySpeedY:Number = _pan_speed*myPercentY
			//
			panToPosition(_pan_target_x + mySpeedX, _pan_target_y + mySpeedY)
		}
		protected function endMousePan(e:MouseEvent):void {
			removeEventListener(Event.ENTER_FRAME,mousePan)
		}
		//
		//***************************************************************
		//
		//
		// HELPERS
		//
		//
		//***************************************************************
		protected function clamp(thisNumber:Number, thisMin:Number = NaN, thisMax:Number = NaN):Number {
			thisNumber = isNaN(thisMin)?thisNumber:Math.max(thisNumber, thisMin)
			thisNumber = isNaN(thisMax)?thisNumber:Math.min(thisNumber, thisMax)
			return thisNumber
		}
		//
		//***************************************************************
		//
		//
		// GETTERS/SETTERS -shortcuts
		//
		//
		//***************************************************************
		public function get foreground():Bitmap {
			return _layers[0]
		}

	}
	
}
